; ##############################################################################
; ##############################################################################
; ##############################################################################
;																			   	
; VERSION: 2016/01																
;																			   	
; SCL(Hz) = CPU(Hz) / (16 + 2 * TWBR * 4^TWPS)									
; Beispiel:																		
; 100.000 = 16.000.000 / (16 + 2 * 18 * 4^1)									
;																			   	
TWI_INI:
; TWI MASTER EINSTELLUNGEN						
.equ TWI_BIT_RATE				= 25				; 255	; Bit-Rate							
.equ TWI_PRESCALER				= 1					; 1		; Vorteiler: 0=1 1=4 2=16 3=64		

	; TWI TWBR 				Bit RATE Register	
	ldi temp, TWI_BIT_RATE
	out TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, TWI_PRESCALER
	out TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	clr temp
	out TWAR,temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|0<<TWIE
	out TWCR,temp
	; PULL-UP ein								
	sbi PORTC,4
	sbi PORTC,5
	rcall WAIT_1ms
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; VERSION: 2016/01																
;																			   	
; START-ROUTINE	(MASTER TRANSMITTER)										   	
;																			   	
; INP: temp1 	(Adresse SLAVE)													
; OUT: R0 		(FEHLER_CODE)													
;																			   	
TWI_MASTER_TRANSMITTER_START_TEMP1:
	WDR
	push temp								; SICHERUNGSKOIE	
	push temp1								; SICHERUNGSKOIE	
	clr R0									; FEHLER lschen	
;-------------------------------------------------------------------------------
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN); START								
	out TWCR, temp	
	rcall TWI_TWINT_wait					; warten, bis TWINT wieder = 1	
	in temp,TWSR							; Check TWSR					
	andi temp, 0xF8
	cpi temp,  0x08							; 0x08 START erfolgreich		
	breq TWI_MASTER_TRANSMITTER_START_w
	cpi temp,  0x10							; 0x10 (repeated START)			
	breq TWI_MASTER_TRANSMITTER_START_w
	rjmp TWI_MASTER_TRANSMITTER_START_ERROR	
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_w:
	LSL  temp1								; Load SLA + "W"  (W=0)  1 x links	
	out TWDR, temp1
	ldi temp, (1<<TWINT) | (1<<TWEN)		; senden - Clear TWINT 				
	out TWCR, temp
	rcall TWI_TWINT_wait					; warten, bis TWINT wieder = 1		
	in temp,TWSR							; Check TWSR						
	andi temp, 0xF8
	cpi temp,  0x18							; 0x18								
	breq TWI_MASTER_TRANSMITTER_START_FERTIG
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_ERROR:
	mov R0, EINS
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_FERTIG:
	pop temp1								; wieder herstellen	
	pop temp								; wieder herstellen	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; VERSION: 2016/01																
; 																			   	
; Ein BYTE zum SLAVE zu senden										   	   	   	
; 																			   	
; INPUT:	temp1 (BYTE)														
; OUT: 		R0    (FEHLER_CODE)													
; 																			   	
TWI_MASTER_TRANSMITTER_BYTE_TEMP1:
	WDR
	push temp								; SICHERUNGSKOIE	
	push temp1								; SICHERUNGSKOIE	
;-------------------------------------------------------------------------------
	tst R0									; Fehler =0 ?	
	brne TWI_MASTER_TRANSMITTER_BYTE_FERTIG
	out TWDR, temp1							; 	Load DATA						
	ldi temp, (1<<TWINT) | (1<<TWEN)		; senden - Clear TWINT 				
	out TWCR, temp
	rcall TWI_TWINT_wait					; warten, bis TWINT wieder = 1		
	in temp,TWSR							; Check TWSR						
	andi temp, 0xF8
	cpi temp, 0x28						
	breq TWI_MASTER_TRANSMITTER_BYTE_FERTIG
;-------------------------------------------------------------------------------
	cbi (LED_ROT_PORT),(LED_ROT_PIN_NR)			; ROT	
	mov R0, EINS
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_BYTE_FERTIG:
	pop temp1								; wieder herstellen	
	pop temp								; wieder herstellen	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; VERSION: 2016/01																
;																			   	
; TWI/I2C START-ROUTINE														   	
;																			   	
; 	INPUT:	temp1 	(ADRESSE SLAVE)											   	
;	OUT: 	R0 		(FEHLER-CODE)   											
;																			   	
TWI_MASTER_RECEIVER_START_TEMP1:
	WDR
	clr R0
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN )	; START-MR 				
	out TWCR, temp	
	rcall TWI_TWINT_wait									; warten, bis TWINT wieder=1
	in temp,TWSR											; Check TWSR 			
	andi temp, 0xF8
	cpi temp,  0x08 										; 0x08 START erfolgreich
	breq TWI_MASTER_RECEIVER_START_RUN
	cpi temp,  0x10											; 0x10 (repeated START)	
	breq TWI_MASTER_RECEIVER_START_RUN
	rjmp TWI_MASTER_RECEIVER_ERROR
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_START_RUN:
	LSL  temp1												; Load SLA +"R" (R=1) 1 x links	
	ori temp1, 0b00000001 									; R=1		
	out TWDR, temp1
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )				; senden - Clear TWINT 			
	out TWCR, temp
	rcall TWI_TWINT_wait									; warten, bis TWINT wieder = 1	
	in temp,TWSR											; Check TWSR					
	andi temp, 0xF8
	cpi temp, 0x40 											;SLA+R gesendet; ACK empfangen
	brne TWI_MASTER_RECEIVER_ERROR
	rjmp TWI_MASTER_RECEIVER_FERTIG
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_ERROR:
	cbi (LED_ROT_PORT),(LED_ROT_PIN_NR)			; ROT	
	mov R0, EINS
TWI_MASTER_RECEIVER_FERTIG:

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; VERSION: 2016/01																
;																				
; Achtung, diese Routine verwenden, wenn 										
; mehrere! BYTES empfangen werden sollen										
;																				
; 	OUT: 	temp1																
;	OUT: 	R0    (FEHLER_CODE)													
;																				
TWI_MASTER_RECEIVER_BYTE_TEMP1:
	WDR
	tst R0 										; ERROR ?	
	breq TWI_MASTER_RECEIVER_BYTE_RUN
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_BYTE_RUN:
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )	; senden "ACK" (TWEA=1) fr mehrere Daten-Bytes 	
	out TWCR, temp
	rcall TWI_TWINT_wait						; TWINT abfragen									
	in temp,TWSR 								; Check TWSR										
	andi temp, 0xF8
	cpi temp, 0x50								; Data Byte empfangen- ACK zurckgesendet			
	brne TWI_MASTER_RECEIVER_BYTE_ERROR
	in temp1, TWDR								; BYTE einlesen										
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_BYTE_ERROR:
	cbi (LED_ROT_PORT),(LED_ROT_PIN_NR)			; ROT	
	clr temp1
	mov R0,EINS	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; VERSION: 2016/01																
;																				
; Achtung, diese Routine verwenden, wenn 										
; nur ein! BYTE empfangen werden soll, oder wenn								
; das letzte! BYTE empfangen werden soll										
;																				
; 	OUT: temp1																	
; 	     R0 (FEHLER_CODE)														
;																				
TWI_MASTER_RECEIVER_LAST_BYTE_TEMP1:
	tst R0 						; ERROR	?
	breq TWI_MASTER_RECEIVER_LAST_BYTE_RUN
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_LAST_BYTE_RUN:
	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )	; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte
	out TWCR, temp
	rcall TWI_TWINT_wait	; TWINT abfragen									
	in temp,TWSR			; Check TWSR										
	andi temp, 0xF8
	cpi  temp, 0x58			; letztes Byte empfangen-NOT ACK zurckgesendet		
	brne TWI_MASTER_RECEIVER_LAST_BYTE_ERROR
	; DATEN einlesen		
	in temp1, TWDR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_LAST_BYTE_ERROR:
	; ERROR								
	clr temp1
	mov R0, EINS	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 																			   	
; STOP- Routine																   	
; 																			   	
TWI_STOP:
	; kleine Pause
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|0<<TWIE	; STOP								
	out TWCR, temp	
	sbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB aus	
	; kleine Pause
	rcall wait_1ms
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TWI_AUS:
	ldi temp, 1<<TWINT|0<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|0<<TWEN|0<<TWIE
	out TWCR,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 																			   	
; warten, bis das TWINT Flag gesetzt ist									   	
; 																			   	
TWI_TWINT_wait:
	in temp, TWCR
	sbrs temp, TWINT	; nchsten Befehl berspringen, wenn TWINT FLAG  gesetzt ist
	rjmp TWI_TWINT_wait	
ret
;###############################################################################
;###############################################################################
;###############################################################################

