;																
; 1-WIRE-SLAVE (HYT939)											
; simuliert den DS18B20											
; ATmega8														
; Bernhard.Erfurt@gmx.de										
; 01/2016														
;																
; https://www.mikrocontroller.net/topic/386142#new				
;																

;																
; Besonderheiten:												
; - verschachtelte Interrupts									
;																

;																
; GRN leuchtet: OK-Lebenslicht									
; GELB leuchtet: MESSUNG aktiv									
; ROT  leuchtet: ERROR											
;																
; Blinken die LEDs, das ist nicht gut !							
; ROT blinkt: 			 Fehler im Programmcheck				
; ROT+GELB+GRN blinken: unkorrekter Interruptaufruf			
;																


;												
; Anschlussbelegung PINS						
;												
; PB0 ==> 										
; PB1 ==> 										
; PB2 ==> 										
; PB3 ==> 										
; PB4 ==> 										
; PB5 ==> 										
; PB6 ==> (Quarz)								
; PB7 ==> (Quarz)								
;												
; PC0 ==> LED GRN								
; PC1 ==> LED GELB								
; PC2 ==> LED ROT								
; PC3 ==> 										
; PC4 ==> SDA									
; PC5 ==> SCL									
;												
; PD0 ==> 										
; PD1 ==> 										
; PD2 ==> 1-WIRE-BUS							
; PD3 ==> 										
; PD4 ==> 										
; PD5 ==> LED GRN	1-WIRE SLAVE RX	ROM-COMMAND	
; PD6 ==> LED GELB 	1-WIRE SLAVE RX	FUNC-COMMAND
; PD7 ==> LED ROT	1-WIRE ERROR				
;												


; ATmega8 										
.include "m8def.inc"
; KONSTANTEN									
.equ TAKT									=4000000		; 4MHz	
; TIMER-WERTE										
.equ TIMERWERT1								=65536-15625	;(2 hoch 16 minus....15625) bei 4MHz + Vorteiler 256 => 1s 		
; Programm-VERSION								
.equ VERSION_TAG							= 16
.equ VERSION_MONAT							= 7
.equ VERSION_JAHR							= 12
.equ VERSION_WOCHENTAG						= 1
.equ VERSION_MESZ							= 1
; 1-WIRE-ROM-ID	(Prfsumme wird automatisch korrigiert)		
.equ ROM_ID_BYTE1							= 0x28		; FAMILY xEF=PRIVAT x28=DS18B20 x10=DS18s20 x27=DS2417(RTC)
.equ ROM_ID_BYTE2							= 0x00
.equ ROM_ID_BYTE3							= 0x20
.equ ROM_ID_BYTE4							= 0x16
.equ ROM_ID_BYTE5							= 0x00
.equ ROM_ID_BYTE6							= 0x00
.equ ROM_ID_BYTE7							= 0x00
.equ ROM_ID_BYTE8							= 0xD7		; Prfsumme				
; 1-WIRE-ROM-COMMANDS							
.equ OW_COMMAND_READ_ROM					= 0x33		; READ-ROM				
.equ OW_COMMAND_MATCH_ROM					= 0x55		; MATCH-ROM				
.equ OW_COMMAND_SEARCH_ROM					= 0xF0		; SEARCH-ROM			
.equ OW_COMMAND_ALARM_SEARCH				= 0xEC		; ALARM-SEARCH			
.equ OW_COMMAND_SKIP_ROM					= 0xCC		; SKIP-ROM				
; 1-WIRE-FUNCTION-COMMANDS						
.equ OW_FUNCTION_CONVERT_T					= 0x44		; CONVERT "T" 			
.equ OW_FUNCTION_RECALL_EE					= 0xB8		; RECALL EEPROM, recalls the alarm trigger values (TH and TL) 		
.equ OW_FUNCTION_READ_SCRATCHPAD			= 0xBE		; read  SCRATCHPAD		
.equ OW_FUNCTION_WRITE_SCRATCHPAD			= 0x4E		; write SCRATCHPAD		
.equ OW_FUNCTION_READ_SCRATCHPAD_PRIVATE	= 0xF0		; read  PRIVATE- DATA	
.equ OW_FUNCTION_WRITE_SCRATCHPAD_PRIVATE	= 0xF6		; write PRIVATE- DATA	
; 1-WIRE-STATUS nicht ndern !!					
.equ OW_STATUS_RX_BEREIT					= 0b00000001; Empfangsbereit								
.equ OW_STATUS_TX_0SLOT						= 0b00000010; eine "0" muss zum MASTER gesendet werden		
.equ OW_STATUS_TX_1SLOT						= 0b00000100; eine "1" muss zum MASTER gesendet werden		
.equ OW_STATUS_GESPERRT						= 0b11110000; GESPERRT, auf RESET-PRESENCE ist wird gewartet
; 1-WIRE-BUS									
.equ OW_PIN_NR						= 2
.equ OW_PORT						= PORTD
.equ OW_DDR							= DDRD
.equ OW_PIN							= PIND
; LED-GRUEN										
.equ LED_GRUEN_PIN_NR				= 0
.equ LED_GRUEN_DDR					= DDRC
.equ LED_GRUEN_PORT					= PORTC
; LED-GELB										
.equ LED_GELB_PIN_NR				= 1
.equ LED_GELB_DDR					= DDRC
.equ LED_GELB_PORT					= PORTC
; LED-ROT										
.equ LED_ROT_PIN_NR					= 2
.equ LED_ROT_DDR					= DDRC
.equ LED_ROT_PORT					= PORTC
; 1-WIRE-LED (GRUEN)							
.equ LED_OW_GRUEN_PIN_NR			= 5
.equ LED_OW_GRUEN_DDR				= DDRD
.equ LED_OW_GRUEN_PORT				= PORTD
; 1-WIRE-LED (GELB)								
.equ LED_OW_GELB_PIN_NR				= 6
.equ LED_OW_GELB_DDR				= DDRD
.equ LED_OW_GELB_PORT				= PORTD
; 1-WIRE-LED (ROT)								
.equ LED_OW_ROT_PIN_NR				= 7
.equ LED_OW_ROT_DDR					= DDRD
.equ LED_OW_ROT_PORT				= PORTD

; REGISTER										
.def DUMMY_R0			= R0	; wichtige Hilfsregister (mit Vorsicht verwenden !)
.def DUMMY_R1			= R1	; wichtige Hilfsregister (mit Vorsicht verwenden !)

.def OW_BYTE			= R6	; wird nur im OW-ISR genutzt						
.def OW_BIT_NR			= R7	; wird nur im OW-ISR genutzt						

.def SREG_KOPIE			= R8	; 
.def temp_KOPIE			= R9	; 

.def ZL_KOPIE			= R10	; 
.def ZH_KOPIE			= R11	; 
.def STATUS				= R12

.def NULL 				= R13
.def EINS 				= R14
.def VOLL				= R15

.def temp 				= R16
.def temp1	 			= R17
.def temp2 				= R18
.def temp3 				= R19
.def temp4 				= R20
.def temp5 				= R21
.def temp6 				= R22
.def temp7 				= R23
.def temp8 				= R24
.def temp9 				= R25


; SRAM-BELEGUNG ATmega8							
.equ adr_DATEN						=SRAM_START			; 0x0060 START Adresse 		

.equ adr_ss							=SRAM_START+31		; Sekunden					
.equ adr_mm							=SRAM_START+32		; Minuten					
.equ adr_hh							=SRAM_START+33		; Stunden					
.equ adr_TAG						=SRAM_START+34		; TAG						
.equ adr_WOCHENTAG					=SRAM_START+35		; Wochentag (1=Mo 7=So)		
.equ adr_MONAT						=SRAM_START+36		; Monat						
.equ adr_JAHR						=SRAM_START+37		; JAHR						
.equ adr_MESZ						=SRAM_START+38		; SOMMERZEIT 0=MEZ 1=MESZ	
.equ adr_SONNE_AUFGANG_mm			=SRAM_START+39		; SONNENAUFGANG				
.equ adr_SONNE_AUFGANG_hh			=SRAM_START+40		; SONNENAUFGANG				
.equ adr_SONNE_UNTERGANG_mm			=SRAM_START+41		; SONNENUNTERGANG			
.equ adr_SONNE_UNTERGANG_hh			=SRAM_START+42		; SONNENUNTERGANG			
.equ adr_ANZAHL_KLINGEL				=SRAM_START+43		; KLINGEL				
.equ adr_ANZAHL_ANRUFE				=SRAM_START+44		; ANRUFE				
.equ adr_DRUCK_L					=SRAM_START+45 		; Luftdruck 			
.equ adr_DRUCK_H					=SRAM_START+46
.equ adr_TEMPERATUR_AUSSEN_L		=SRAM_START+47 		; Aussentemperatur		
.equ adr_TEMPERATUR_AUSSEN_H		=SRAM_START+48
.equ adr_TEMPERATUR_RAUM_WZ_L		=SRAM_START+49 		; RAUM WZ				
.equ adr_TEMPERATUR_RAUM_WZ_H		=SRAM_START+50
.equ adr_TEMPERATUR_RAUM_AZ_L		=SRAM_START+51 		; RAUM AZ				
.equ adr_TEMPERATUR_RAUM_AZ_H		=SRAM_START+52
.equ adr_TEMPERATUR_RAUM_KU_L		=SRAM_START+53 		; RAUM Kche			
.equ adr_TEMPERATUR_RAUM_KU_H		=SRAM_START+54
.equ adr_TEMPERATUR_RAUM_BA_L		=SRAM_START+55 		; RAUM BAD				
.equ adr_TEMPERATUR_RAUM_BA_H		=SRAM_START+56
.equ adr_TEMPERATUR_RAUM_FL_L		=SRAM_START+57 		; RAUM Flur				
.equ adr_TEMPERATUR_RAUM_FL_H		=SRAM_START+58
.equ adr_TEMPERATUR_RAUM_TH_L		=SRAM_START+59 		; RAUM Treppenhaus		
.equ adr_TEMPERATUR_RAUM_TH_H		=SRAM_START+60
.equ adr_TEMPERATUR_HZG_WZ_L		=SRAM_START+61 		; HEIZUNG WZ			
.equ adr_TEMPERATUR_HZG_WZ_H		=SRAM_START+62
.equ adr_TEMPERATUR_HZG_AZ_L		=SRAM_START+63 		; HEIZUNG AZ			
.equ adr_TEMPERATUR_HZG_AZ_H		=SRAM_START+64
.equ adr_TEMPERATUR_HZG_KU_L		=SRAM_START+65 		; HEIZUNG Kche			
.equ adr_TEMPERATUR_HZG_KU_H		=SRAM_START+66
.equ adr_TEMPERATUR_HZG_BA_L		=SRAM_START+67 		; HEIZUNG BAD			
.equ adr_TEMPERATUR_HZG_BA_H		=SRAM_START+68
.equ adr_TEMPERATUR_HAUS_KW_L		=SRAM_START+69 		; HAUS Kaltwasser		
.equ adr_TEMPERATUR_HAUS_KW_H		=SRAM_START+70
.equ adr_TEMPERATUR_HAUS_WW_L		=SRAM_START+71 		; HAUS Warmwasser		
.equ adr_TEMPERATUR_HAUS_WW_H		=SRAM_START+72
.equ adr_TEMPERATUR_HAUS_HZG_L		=SRAM_START+73 		; HAUS HZG				
.equ adr_TEMPERATUR_HAUS_HZG_H		=SRAM_START+74
.equ adr_TEMPERATUR_AUSSEN1_L		=SRAM_START+75 		; AUSSEN-1 	(WZ)		
.equ adr_TEMPERATUR_AUSSEN1_H		=SRAM_START+76
.equ adr_TEMPERATUR_AUSSEN2_L		=SRAM_START+77 		; AUSSEN-2	(AZ)		
.equ adr_TEMPERATUR_AUSSEN2_H		=SRAM_START+78
.equ adr_TEMPERATUR_AUSSEN3_L		=SRAM_START+79 		; AUSSEN-3	(K)		
.equ adr_TEMPERATUR_AUSSEN3_H		=SRAM_START+80
.equ adr_TEMPERATUR_RESERVE4_L		=SRAM_START+81 		; RESERVE-4				
.equ adr_TEMPERATUR_RESERVE4_H		=SRAM_START+82
.equ adr_CONTROLL_BYTE_1			=SRAM_START+83		; KONTROLLBYTE			
.equ adr_CONTROLL_BYTE_2			=SRAM_START+84		; KONTROLLBYTE			


.equ adr_HYT939_VORHANDEN			=SRAM_START+140
.equ adr_HYT939_DATA_MSB			=SRAM_START+141
.equ adr_HYT939_DATA_LSB			=SRAM_START+142
.equ adr_HYT939_TEMPERATUR_DATA_MSB	=SRAM_START+143
.equ adr_HYT939_TEMPERATUR_DATA_LSB	=SRAM_START+144
.equ adr_HYT939_RH					=SRAM_START+145		; ERGEBNIS %			
.equ adr_HYT939_RH_OLD				=SRAM_START+146		; ERGEBNIS % (OLD)		
.equ adr_HYT939_TEMPERATUR_SIGNIERT	=SRAM_START+147		; ERGEBNIS Temperatur	


.equ adr_OW_ZAEHLER_RESET_PRESENCE_L=SRAM_START+400		; fr Wartungszwecke (ZAEHLERWERTE)	
.equ adr_OW_ZAEHLER_RESET_PRESENCE_H=SRAM_START+401		; 									

.equ adr_OW_FREIGABE_MESSUNG		=SRAM_START+402		; MESSUNG aktivieren				

.equ adr_OW_ERROR_COMMAND			=SRAM_START+403
.equ adr_OW_ERROR_FUNCTION			=SRAM_START+404

.equ adr_OW_BIT_NR					=SRAM_START+410		; Zhler							
.equ adr_OW_BYTE_ZAEHLER			=SRAM_START+411		; Zhler							

.equ adr_OW_ROM_MATCH_ZAEHLER		=SRAM_START+412		; Zhler							

.equ adr_OW_ROM_SEARCH_ZAEHLER_AB	=SRAM_START+420		; Zhler							
.equ adr_OW_ROM_SEARCH_BIT_ZAEHLER	=SRAM_START+421		; Zhler							
.equ adr_OW_ROM_SEARCH_ID_BYTE_NR	=SRAM_START+422		;
.equ adr_OW_ROM_SEARCH_BYTE			=SRAM_START+423		;


.equ adr_OW_ROM_COMMAND				=SRAM_START+424		; ROM COMMAND 						
.equ adr_OW_ROM_COMMAND_GESPERRT	=SRAM_START+425		; ROM COMMAND abgearbeitet			
.equ adr_OW_FUNCTION_COMMAND		=SRAM_START+426		; FUNCTION COMMAND					
.equ adr_OW_FUNCTION_COMMAND_GESPERRT=SRAM_START+427	; FUNCTION COMMAND abgearbeitet		



.equ adr_OW_ROM_ID_BEREICH8			=SRAM_START+500		; 8 BYTE ROM-ID SLAVE				

.equ adr_OW_SCRATCHPAD_BEREICH256	=SRAM_START+600		; SCRATCHPAD  (256 BYTE)			


.equ adr_SRAM_CHECK					=SRAM_START+943		; SRAM CHECK muss immer "S" sein	



.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0

rjmp RESET      		; Reset Handler                           				
rjmp EXT_INT0 		  	; IRQ0 Handler     										
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM2_OVF   ; Timer2 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp TIM1_OVF   	  	; Timer1 Overflow Handler       						
rjmp INTERRUPT_ERROR	;rjmp TIM0_OVF   ; Timer0 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp SPI_STC    ; SPI Transfer Complete Handler     	
rjmp INTERRUPT_ERROR	;rjmp ZEICHEN_EINLESEN 	; USART RX Complete Handler     
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADC        ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI       ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	
; Unkorrekter Interrupt Aufruf 													
INTERRUPT_ERROR:
	CLI											; INTERRUPTS sperren	
	sbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN aus	
	sbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB aus	
	sbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  aus	
	rcall wait_100ms
	cbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN an	
	cbi (LED_GELB_PORT),(LED_GELB_PIN_NR)		; GELB an	
	cbi (LED_ROT_PORT),(LED_ROT_PIN_NR)			; ROT  an	
	rcall wait_100ms
	rjmp INTERRUPT_ERROR

RESET:                        
; STACK initialisieren								
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; WDR_INITIALISIERUNG								
	rcall WDR_INITIALISIERUNG
; PORT_CLEAR										
	rcall PORT_CLEAR
; LED_INITIALISIERUNG								
	rcall LED_INITIALISIERUNG
; NEUSTART_MIT_WDR									
	rcall NEUSTART_MIT_WDR
; LED-AUS											
	rcall LED_AUS
; SRAM lschen										
	rcall SRAM_CLEAR
; REGISTER_CLEAR									
	rcall REGISTER_CLEAR
; STARTWERTE										
	; NULL				
	clr NULL
	; EINS				
	ldi temp,1
	mov EINS,temp
	; VOLL				
	ldi temp,255
	mov VOLL,temp
	; SRAM CHECK		
	ldi temp,'S'
	STS(adr_SRAM_CHECK),temp
	; DATUM				
	ldi temp,(VERSION_TAG)
	STS(adr_TAG),temp
	ldi temp,(VERSION_MONAT)
	STS(adr_MONAT),temp
	ldi temp,(VERSION_JAHR)	
	STS(adr_JAHR),temp
	ldi temp,(VERSION_WOCHENTAG)	
	STS(adr_WOCHENTAG),temp
	; ZEIT				
	ldi temp,45
	STS(adr_ss),temp
	ldi temp,59
	STS(adr_mm),temp
	ldi temp,23
	STS(adr_hh),temp
	ldi temp,VERSION_MESZ
	STS(adr_MESZ),temp
; 1-WIRE_INITIALISIERUNG							
	rcall ONE_WIRE_INITIALISIERUNG
; 1-WIRE-EXTERNER_INTERUPT_INITIALISIEREN			
	rcall EXTERNER_INTERUPT_INITIALISIEREN
; 1-WIRE-EXTERNER_INTERRUPT_AKTIVIEREN				
	rcall EXTERNER_INTERRUPT_AKTIVIEREN						
; TIMER1_INITIALISIERUNG							
	rcall TIMER1_INITIALISIERUNG
; TIMER_AKTIVIEREN									
	rcall TIMER_AKTIVIEREN
; 	PROGRAMM-CHECK									
	rcall PROGRAMM_CHECK
; ##############################################################################
; Interrupt															
	out GIFR,VOLL			; eventuelle Pegelwechsel ignorieren	
	SEI						; interrupts freigeben					
	cbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR	; GRN		

	rcall TWI_INI
	rcall TWI_DISPLAY_INITIALISIERUNG
; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	WDR									; Watch-Dog-RESET	
	SLEEP
	rcall MESSUNG
	rcall SCRATCHPAD_CRC8
	rcall EXT_DISPLAY_HAUPTMENUE
	rcall PROGRAMM_CHECK
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
MESSUNG:
	LDS temp,(adr_OW_FREIGABE_MESSUNG)
	tst temp
	brne MESSUNG_RUN
	ret
;-------------------------------------------------------------------------------
MESSUNG_RUN:
	cbi (LED_GELB_PORT),(LED_GELB_PIN_NR)		; GELB 	
	STS(adr_OW_FREIGABE_MESSUNG),NULL
	rcall HYT939_MESSUNG
	rcall MESSUNG_UMWANDLUNG_AUF_DS18B20
	sbi (LED_GELB_PORT),(LED_GELB_PIN_NR)		; GELB 	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; SCRATCHPAD:																	
; byte 0 Temperature LSB (50h)													
; byte 1 Temperature MSB (05h)													
; byte 2 TH Register or User Byte 1												
; byte 3 TL Register or User Byte 2												
; byte 4 Configuration Register													
; byte 5 Reserved (FFh)															
; byte 6 Reserved (0Ch)															
; byte 7 Reserved (10h)															
; byte 8 CRC*																	
; 																				
; CONFIGURATION REGISTER;														
; 0 R1 R0 1 1 1 1 1																
; R1 R0 Resolution Max Conversion Time											
; 0 0 	 9-bit																	
; 0 1 	10-bit																	
; 1 0 	11-bit																	
; 1 1 	12-bit																	
; 																				
; BEISPIELDATEN:
; +125C  		0x07D0 =															
; +85C*  		0x0550 = 1360 = 0b00000101-01010000									
; +25.0625C  	0x0191 =															
; +10.125C  	0x00A2 =															
; +0.5C  		0x0008 = 8															
; 0C  			0x0000 = 0															
; -0.5C  		0xFFF8 =															
; -10.125C  	0xFF5E =															
; -25.0625C  	0xFE6F =															
; -55C  		0xFC90 =															
; 																					
MESSUNG_UMWANDLUNG_AUF_DS18B20:
	

	LDS temp1,(adr_HYT939_RH)
	clr temp2
	; 4 x links				
	LSL temp1
	ROL temp2

	LSL temp1
	ROL temp2

	LSL temp1
	ROL temp2

	LSL temp1
	ROL temp2

	STS(adr_OW_SCRATCHPAD_BEREICH256+0),temp1
	STS(adr_OW_SCRATCHPAD_BEREICH256+1),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "SLAVE_1WIRE_INI.asm"				; 1-wire-INITIALISIERUNG			
.include "SLAVE_1WIRE_INTERRUPT.asm"		; 1-wire-INTERRUPTS					
.include "SLAVE_1WIRE_ROM_COMMAND.asm"		; 1-wire ROM-COMMANDS				
.include "SLAVE_1WIRE_FUNCTION_COMMAND.asm" ; 1-wire FUNKTION-COMMANDS			
.include "SLAVE_SONSTIGES.asm"				; sonstiges							
.include "SLAVE_TIMER.asm"					; TIMER								
.include "SLAVE_EXT_INTERRUPT.asm"			; ext. INTERRUPT					
.include "SLAVE_TWI_MASTER.asm"				; TWI_MASTER						
.include "SLAVE_TWI_HYT939.asm"				; TWI
.include "SLAVE_TWI_LCD.asm"				; ext.LCD-DISPLAY (WARTUNGSZWECKE)	
.include "SLAVE_WAIT_04MHZ.asm"				; 4 MHz								
; ##############################################################################
; ##############################################################################
; ##############################################################################

